# System imports
from flask_login import LoginManager, current_user, login_required, login_user, logout_user
from datetime import datetime
from flask import *
import bcrypt
import json
import os

# Custom imports
try:
    from .corn import *
except ImportError:
    from corn import *

# Variables
app = Flask(__name__)
app.secret_key = os.urandom(24)
users_dict = {}
login_manager = LoginManager()
login_manager.login_view = 'login'
login_manager.init_app(app)
last_clear_time = datetime.now()
app.config['SESSION_COOKIE_SAMESITE'] = "None"


# Login manager setup
@login_manager.user_loader
def load_user(user_id):
    if user_id in users_dict:
        return users_dict[user_id]
    else:
        return None


# The home page, a user can create and view their notes here.
@app.route('/')
@login_required
def home():
    return render_template('home.html', user=current_user.name, notes=users_dict[current_user.name].notes)


# Authorize the user.
@app.route('/login', methods=['GET', 'POST'])
def login():
    if request.method == "POST":
        if "username" in request.form and "password" in request.form:
            if request.form.get('username') in users_dict:
                if bcrypt.checkpw(request.form.get('password').encode(),
                                  users_dict[request.form.get('username')].password):
                    login_user(users_dict[request.form.get('username')])
                    return redirect(url_for('home'))
                else:
                    flash("Invalid credentials.")
                    return redirect(url_for('login'))
            else:
                flash("User not found.")
                return redirect(url_for('login'))
        else:
            return "Missing parameters."
    if request.method == "GET":
        if current_user.is_authenticated:
            return redirect(url_for('home'))
        return render_template('login.html')


# Add a new account to the website.
@app.route('/signup', methods=['GET', 'POST'])
def signup():
    if request.method == "POST":
        if "username" in request.form and "password" in request.form:
            if request.form.get('username') not in users_dict:
                users_dict[request.form.get('username')] = users.User(
                    request.form.get('username'),
                    bcrypt.hashpw(request.form.get('password').encode(), bcrypt.gensalt()))
                return redirect(url_for("login"))
            else:
                flash("User already exists.")
                return redirect(url_for('signup'))
        else:
            return "Missing parameters."
    if request.method == "GET":
        return render_template('signup.html')


# Create a new note.
@app.route('/create_note', methods=['POST'])
@login_required
def create_note():
    if "title" not in request.form or "note" not in request.form:
        return "Missing parameters."
    if request.form.get('title') in users_dict[current_user.name].notes.keys():
        flash("Note name already taken.")
        return redirect(url_for('home'))
    users_dict[current_user.name].notes[request.form.get('title').replace('\\', '')[:50]] = \
        request.form.get('note').replace('\\', '')
    return redirect(url_for('home'))


# Export the notes as JSON.
@app.route('/export_notes')
@login_required
def export_notes():
    if len(users_dict[current_user.name].notes) == 0:
        return "{}"
    # Very important to set the mimetype, otherwise the browser will try to guess it and we end up with an xss.
    return parse_json.parse_json(
        Response(json.dumps(users_dict[current_user.name].notes, sort_keys=True), mimetype="text")
    )


# Log the user out.
@app.route('/logout')
@login_required
def logout():
    logout_user()
    return redirect(url_for('login'))


# Remove admin logs
@app.route('/clear')
@login_required
def clear():
    global last_clear_time
    current_time = datetime.now()
    time_difference = current_time - last_clear_time
    if time_difference.seconds > 10:
        if current_user.name == 'admin':
            users_dict['admin'].notes = {'flag': "CTF{REDACTED}"}
        else:
            users_dict[current_user.name].notes = {}
        flash("Notes cleared.")
        last_clear_time = current_time
        return redirect(url_for('home'))
    else:
        flash("Notes can only be cleared once every 10 seconds.")
        return redirect(url_for('home'))


# Admin setup.
users_dict['admin'] = users.User("admin", bcrypt.hashpw(b"REDACTED", bcrypt.gensalt()))
users_dict['admin'].notes['flag'] = "CTF{REDACTED}"


def main():
    app.run(host='0.0.0.0')


if __name__ == '__main__':
    main()
